library(tidyverse)
library(lubridate)
library(serial)

# See what ports are available:
listPorts()

# Change the port here (COM1, etc.)
#
# Only change 4800 in the mode if you know for sure the NMEA stream is using a
# different (and non-standard) baud rate
con <- serialConnection(
  port = "ttyUSB0", 
  mode = "4800,n,8,1"
)
open(con)

# Read from the port for 60 seconds or once we've received 100 lines of data
num_lines = 0
time_seconds = 0
all_lines <- c()
while (num_lines < 100 & time_seconds < 60) {
    data <- read.serialConnection(con)
    lines <- str_split(data, "\\$")[[1]]
    lines <- subset(lines, grepl("^GP", lines))
    print(lines)
    num_lines <<- num_lines + length(lines)
    time_seconds <<- time_seconds + 1
    all_lines <<- c(all_lines, lines)
    Sys.sleep(1)
}

close(con)

# Was there data from the port?
if (num_lines == 0) {
  print("Error: no data resembling NMEA on this port")
} else {
  # Do we have NMEA position data (GPGGA)
  nmea_data <- tibble(raw = all_lines) %>%
    filter(grepl("GPGGA", raw))

  if (nrow(nmea_data) == 0) {
    print("Error: no NMEA data")
  } else {
    # Does the data contain GPS coordinates?
    nmea_data <- nmea_data %>%
      mutate(
        fields = str_split(raw, ","),
        hhmmss = map_chr(fields, ~ pluck(., 2)),
        latddmm = map_chr(fields, ~ pluck(., 3)),
        ns = map_chr(fields, ~ pluck(., 4)),
        londdmm = map_chr(fields, ~ pluck(., 5)),
        ew = map_chr(fields, ~ pluck(., 6)),
      ) %>%
      filter(
        str_length(latddmm) > 4,
        str_length(londdmm) > 5
      )
    if (nrow(nmea_data) == 0) {
      print("WARNING: no valid locations in data")
    } else {
      nmea_data <- nmea_data %>%
        mutate(
          ts_utc = strptime(paste(date(now(tzone = "UTC")), hhmmss), "%Y-%m-%d %H%M%OS", tz = "UTC"),
          latdd = as.numeric(gsub("([0-9]{2}).*", "\\1", latddmm)),
          latmm = as.numeric(gsub("[0-9]{2}([0-9.]+)", "\\1", latddmm)),
          lat = if_else(ns == "S", -1 * latdd + latmm / 60.0, latdd + latmm / 60.0),
          londd = as.numeric(gsub("([0-9]{3}).*", "\\1", londdmm)),
          lonmm = as.numeric(gsub("[0-9]{3}([0-9.]+)", "\\1", londdmm)),
          lon = if_else(ns == "W", -1 * londd + lonmm / 60.0, londd + lonmm / 60.0)
        ) %>%
        select(ts_utc, lat, lon, raw)

      # Print the 10 most recent locations
      print(nmea_data %>% tail(10))
    }
  }
}
